package examples.org.servicetango.service.http;

import org.osgi.framework.*;
import org.osgi.service.http.*;
import org.osgi.service.log.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;

import org.servicetango.core.*;

public class HelloWorldBundle implements BundleActivator, ServiceListener
{
    BundleContext bc = null;
    HttpService httpSvc = null;
    private ServiceReference httpRef = null;
    LogService logSvc = null;
    private ServiceReference logRef = null;

    private static String httpsvcClass = "org.osgi.service.http.HttpService";
    private static String logsvcClass = "org.osgi.service.log.LogService";

    HttpContext httpContext;
    String servletURL = "/hello";

    /**
     * Must have a public default constructor!
     * It doesn't have to do anything but this class must be
     * able to be instantiated by Class.newInstance()
     * This same instance that of this object that starts the bundle
     * will be used to stop the bundle.
     */
    public HelloWorldBundle()
    {
        httpContext = new HelloWorldContext();
    }

    /**
     * Part of the BundleActivator interface. This method starts
     * the bundle and must quickly return. If the bundle requires
     * its own threads. they can be created here. If the bundle
     * registers any Service, they can be registered here.
     *
     * @param   bc BundleContext object for this bundle
     */
    public void start(BundleContext bc)
    throws BundleException
    {
        this.bc = bc;

        getLogService();
        registerServlet();

        bc.addServiceListener(this);
    }

    /**
     * Part of the BundleActivator interface. This method stops
     * the bundle and must quickly return. If the bundle registered any
     * Services they are already unregister before this method is called.
     * This same instance that of this object that starts the bundle
     * will be used to stop the bundle.
     *
     * @param   bc BundleContext object for this bundle
     */
    public void stop(BundleContext bc)
    {
        bc.removeServiceListener(this);

        unregisterServlet();
        ungetLogService();

        this.bc = null;
    }


    // Implementation of ServiceListener

    public void serviceChanged(ServiceEvent evt) {
        try {
            ServiceReference svcref = evt.getServiceReference();
            String[] objectClass = (String[])svcref.getProperty("objectClass");
            for (int i = 0; i < objectClass.length; i++) {
                if (objectClass[i].equals(logsvcClass)) {
                    switch (evt.getType()) {
                    case ServiceEvent.REGISTERED:
                        getLogService();
                        return;
                    case ServiceEvent.UNREGISTERING:
                        ungetLogService();
                        return;
                    }
                } else if (objectClass[i].equals(httpsvcClass)) {
                    switch (evt.getType()) {
                    case ServiceEvent.REGISTERED:
                        registerServlet();
                        return;
                    case ServiceEvent.UNREGISTERING:
                        unregisterServlet();
                        return;
                    }
                }
            }
        } catch(Exception e) {
            //ignore
        }
    }

    synchronized void registerServlet()
    throws BundleException
    {
        if (httpSvc == null) {
            httpRef = bc.getServiceReference(httpsvcClass);

            if (httpRef != null) {
                httpSvc = (HttpService)bc.getService(httpRef);

              if (httpSvc != null) {
                  try {
                      httpSvc.registerServlet(servletURL, new HelloWorld(this),
                                              null, httpContext);
                  } catch (Exception e) {
                        throw new STException("Failed to register servlet", e);
                  }
              }
              else
                throw new BundleException("Failed to get an instance of service class "+httpsvcClass);
            }
            else
              throw new BundleException("This bundle requires service class "+httpsvcClass);
        }
    }

    synchronized void unregisterServlet()
    {
        if (httpSvc != null) {
            try {
                httpSvc.unregister(servletURL);
            } catch (Exception e) {
                logError(HelloWorldResource.unregSvltErr(), e);
            }

            if (httpRef != null) {
                bc.ungetService(httpRef);
                httpRef = null;
            }
            httpSvc = null;
        }
    }

    synchronized void getLogService()
    throws BundleException
    {
        if (logSvc == null) {
            logRef = bc.getServiceReference(logsvcClass);
            if (logRef == null)
              throw new BundleException("This bundle requires service class "+logsvcClass);
            logSvc = (LogService)bc.getService(logRef);
        }
    }

    synchronized void ungetLogService()
    {
        if (logSvc != null) {
            if (logRef != null) {
                bc.ungetService(logRef);
                logRef = null;
            }
            logSvc = null;
        }
    }

    void logError(String message, Throwable t) {
      if (t == null) {
        logError(message, t);
      }
      else if (logSvc != null)  {
            try {
                logSvc.log(LogService.LOG_ERROR, message, t);
            }
            catch (Exception e) {
            }
        }
    }
    void logError(String message) {
        if (logSvc != null)        {
            try {
                logSvc.log(LogService.LOG_ERROR, message);
            }
            catch (Exception e) {
            }
        }
    }
}
