/*
   STBundleContents.java

   Copyright (C) 2000 by ted stockwell
   All Rights Reserved.
 */
package org.servicetango.core;

import java.util.*;
import java.util.jar.*;
import java.io.*;

import org.osgi.framework.*;

/**
 *  Encapsulates access to a bundle's contents (jar file).
 *
 *  @author   ted stockwell [emorning@mediaone.net]
 */
public class STBundleContents
{
    Manifest m_manifest;
    Map m_entries= new HashMap();
    Map m_content= new HashMap();
    Collection m_exportedPackages= null;
    Collection m_exportedServices= null;

    /**
     * Loads all the given jar file's entries into memory.
     * After invoking this constructor the denoted JarFile may be closed.
     */
    public STBundleContents(JarFile jarFile)
    throws IOException, BundleException
    {
        m_manifest= jarFile.getManifest();

        /*
        if the given jar file does not have a manifest then it is not a valid
        bundle
        */
        if (m_manifest == null)
            throw new STException(jarFile.getName()+" is not a valid bundle, it does not contain a manifest.");

        Enumeration enumEntries= jarFile.entries();
        for (; enumEntries.hasMoreElements();) {
            JarEntry entry= (JarEntry)enumEntries.nextElement();
            String name= entry.getName();
            m_entries.put(name, entry);

            // read the entire contents
            InputStream in= jarFile.getInputStream(entry);
            try {
                byte[] content= loadBytesFromStream(in, (int)entry.getSize());
                m_content.put(name, content);
            }
            finally {
                try { in.close(); } catch (Throwable t) { }
            }
        }
    }

    public Manifest getManifest() {
        return m_manifest;
    }

    public byte[] getEntryData(String entryName) {
        return (byte[])m_content.get(entryName);
    }

    public synchronized Collection getExportedPackageNames() {
        if (m_exportedPackages == null) {
            Attributes attributes= m_manifest.getMainAttributes();
            String exported= (String)attributes.get("Export-Package");
            m_exportedPackages= new ArrayList();
            if (exported == null) {
                StringTokenizer tokenizer= new StringTokenizer(exported, ";,");
                for (; tokenizer.hasMoreTokens();) {
                    String token= tokenizer.nextToken();
                    if (token.indexOf('=') < 0)
                        m_exportedPackages.add(token.trim());
                }
            }
        }
        return m_exportedPackages;
    }
    public Collection getExportedServiceNames() {
        if (m_exportedServices == null) {
            Attributes attributes= m_manifest.getMainAttributes();
            String exported= (String)attributes.get("Export-Service");
            m_exportedServices= new ArrayList();
            if (exported == null) {
                StringTokenizer tokenizer= new StringTokenizer(exported, ";,");
                for (; tokenizer.hasMoreTokens();) {
                    String token= tokenizer.nextToken();
                    if (token.indexOf('=') < 0)
                        m_exportedServices.add(token.trim());
                }
            }
        }
        return m_exportedServices;
    }

    /**
     * Loads all the bytes of an InputStream.
     */
    private static byte[] loadBytesFromStream(InputStream in, int length)
         throws IOException
     {
	byte[] buf = new byte[length];
	int nRead, count = 0;
	while ((length > 0) && ((nRead = in.read(buf,count,length)) != -1)) {
            count += nRead;
            length -= nRead;
	}
        return buf;
    }
}
