/*
   STBundleContext.java

   Copyright (C) 2000 by ted stockwell
   All Rights Reserved.
 */
package org.servicetango.core;

import org.servicetango.util.*;
import org.osgi.framework.*;

import java.io.*;
import java.util.*;


/**
 *  ServiceTango's implementation of the BundleContext interface.
 *
 *  @author   ted stockwell [emorning@mediaone.net]
 */
public class STBundleContext
extends ProfileMap
implements BundleContext
{
    public static final String KEY_BASE= STBundleContext.class.getName()+'.';

    STBundle m_bundle;
    BundleManager m_manager;
    STConfiguration m_configuration;
    STStorage m_storage;

    public STBundleContext(STBundle bundle) {
        m_bundle= bundle;
        m_manager= m_bundle.getBundleManager();
        m_configuration= m_manager.getConfiguration();
        m_storage= m_manager.getStorage();
    }
    protected BundleManager getBundleManager() {
        return m_manager;
    }

    public String getProperty(String key) {
        String value= m_configuration.getSTProperty(key);
        return value;
    }
    public Bundle getBundle() {
        return m_bundle;
    }
    public Bundle installBundle(String location)
    throws BundleException
    {
        Bundle bundle= m_manager.installBundle(location);
        return bundle;
    }
    public Bundle installBundle(String location, InputStream in)
    throws BundleException
    {
        Bundle bundle= m_manager.installBundle(location, in);
        return bundle;
    }
    public Bundle getBundle(long id) {
        Bundle bundle= m_manager.getBundle(id);
        return bundle;
    }
    public Bundle[] getBundles() {
        Bundle[] bundle= m_manager.getBundles();
        return bundle;
    }
    public void addServiceListener(ServiceListener listener, String filter)
    throws InvalidSyntaxException
    {
        m_manager.addServiceListener(m_bundle.getBundleId(), listener, filter);
    }
    public void addServiceListener(ServiceListener listener) {
        m_manager.addServiceListener(m_bundle.getBundleId(), listener);
    }
    public void removeServiceListener(ServiceListener listener) {
        m_manager.removeServiceListener(listener);
    }
    public void addBundleListener(BundleListener listener) {
        m_manager.addBundleListener(m_bundle.getBundleId(), listener);
    }
    public void removeBundleListener(BundleListener listener) {
        m_manager.removeBundleListener(listener);
    }
    public void addFrameworkListener(FrameworkListener listener) {
        m_manager.addFrameworkListener(m_bundle.getBundleId(), listener);
    }
    public void removeFrameworkListener(FrameworkListener listener) {
        m_manager.removeFrameworkListener(listener);
    }
    public ServiceRegistration registerService(String[] clazzes, Object service, Dictionary properties) {
        return m_manager.registerService(m_bundle.getBundleId(), clazzes, service, properties);
    }
    final public ServiceRegistration registerService(String clazz, Object service, Dictionary properties) {
        return registerService(new String[] { clazz }, service, properties);
    }
    public ServiceReference[] getServiceReferences(String clazz, String filter)
    throws InvalidSyntaxException
    {
        return m_manager.getServiceReferences(clazz, filter);
    }
    public ServiceReference getServiceReference(String clazz) {
        return m_manager.getServiceReference(clazz);
    }
    public Object getService(ServiceReference reference) {
        return m_manager.getService(m_bundle.getBundleId(), (STServiceReference)reference);
    }
    public boolean ungetService(ServiceReference reference) {
        return m_manager.ungetService(m_bundle.getBundleId(), (STServiceReference)reference);
    }
    public File getDataFile(String name) {
        return m_storage.getDataFile(m_bundle.getLocation(), name);
    }
}
