/*
   STConfiguration.java

   Copyright (C) 2000 by ted stockwell
   All Rights Reserved.
 */
package org.servicetango.core;

import org.servicetango.util.*;

import javax.xml.parsers.*;

import java.util.*;
import java.io.*;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *  Encapsulates a BundleManager's configuration information.
 *
 *  @author  ted stockwell [emorning@mediaone.net]
 */
public class STConfiguration
{
    static final String KEY_ROOT_DIRECTORY= "root-directory";
    static final String KEY_BUNDLE_TIMEOUT_MINUTES= "bundle-timeout-minutes";
    static final String KEY_LOG_BUNDLE_URL= "log-bundle-url";

    Element m_element= null;
    ElementMap m_map= new ElementMap();

    public STConfiguration(InputStream in)
    throws IOException, SAXException
    {
        load(in);
    }

    public void load(InputStream in)
    throws IOException, SAXException
    {
        m_map.load(in);
    }

    public void save(OutputStream out)
    throws IOException
    {
        m_map.save(out);
    }

    public String getSTProperty(String key) {
        String value= LangUtils.toString(m_map.get(key));
        if (value == null)
            value= LangUtils.toString(DEFAULTS.get(key));
        return value;
    }
    public void setSTProperty(String key, String value) {
        m_map.put(key, value);
    }
    public String[] getSTPropertyKeys() {
        ArrayList keys= new ArrayList(DEFAULTS.keySet());
        keys.addAll(m_map.keySet());
        String[] results= new String[keys.size()];
        keys.toArray(results);
        return results;
    }

    public String getRootDirectory() {
        String value= getSTProperty(KEY_ROOT_DIRECTORY);
        return value;
    }

    public int getBundleTimeoutMinutes() {
        String value= getSTProperty(KEY_BUNDLE_TIMEOUT_MINUTES);
        return LangUtils.toInt(value, 2);
    }

    static final Map DEFAULTS= new HashMap();
    static {
        DEFAULTS.put(KEY_BUNDLE_TIMEOUT_MINUTES, "2");
        DEFAULTS.put("org.osgi.framework.version", "1.0");
        DEFAULTS.put("org.osgi.framework.vendor", "servicetango.org");

        /*
        UNFINISHED - need to check this
        The language being used. See ISO 639 for possible values.
        */
        DEFAULTS.put("org.osgi.framework.language", "en");

        DEFAULTS.put("org.osgi.framework.os.name", System.getProperty("os.name"));
        DEFAULTS.put("org.osgi.framework.os.version", System.getProperty("os.version"));
        DEFAULTS.put("org.osgi.framework.processor", System.getProperty("os.arch"));
        DEFAULTS.put(KEY_ROOT_DIRECTORY, "./stcache");
    }
}
