/*
   STServiceReference.java

   Copyright (C) 2000 by ted stockwell
   All Rights Reserved.
 */
package org.servicetango.core;

import org.servicetango.util.*;
import org.osgi.framework.*;
import java.util.*;
import java.io.*;

/**
 *
 *  @author   ted stockwell [emorning@mediaone.net]
 */
public class STServiceReference
implements ServiceReference, Serializable
{
    BundleManager m_manager;
    Bundle m_bundle;
    Dictionary m_properties;
    String[] m_clazzes;
    Object m_service= null;
    HashMap m_counts= new HashMap();
    HashMap m_services= new HashMap();
    STServiceRegistration m_registration= null;

    public STServiceReference(){
    }

    public void init(BundleManager manager, STServiceRegistration registration, Bundle bundle, String[] clazzes, Object service, Dictionary properties) {
        m_manager= manager;
        m_bundle= bundle;
        m_clazzes= clazzes;
        m_service= service;
        m_registration= registration;
        Map m= CollectionUtils.toMap(properties);
        m_properties= new Hashtable(m);
        m_properties.put("objectClass", m_clazzes);
    }

    public STServiceRegistration getServiceRegistration() {
            return m_registration;
    }

    public void setProperties(Dictionary properties) {
        if (!m_manager.isRegisteredService(this))
            throw new IllegalStateException("Service is not registered");

        Map m= CollectionUtils.toMap(properties);
        m_properties= new Hashtable(m);

        /*
        The key "objectClass" cannot be modified by this method.
        */
        m_properties.put("objectClass", m_clazzes);

        ServiceEvent e= new ServiceEvent(ServiceEvent.MODIFIED, this);
        m_manager.fireServiceEvent(e);
    }

    /**
     * Return a service reference for the given bundle
     */
    synchronized public Object getService(long bundleId) {
        Long id= new Long(bundleId);
        Object service= m_services.get(id);
        if (service == null) {
            if (m_service instanceof ServiceFactory) {
                Bundle bundle= m_manager.getBundle(bundleId);
                service= ((ServiceFactory)m_service).getService(bundle, m_registration);
                m_services.put(id, service);
            }
            else
                service= m_service;
        }
        incrUseCount(bundleId);
        return service;
    }

    /**
     * release a service
     */
     synchronized public boolean ungetService(long bundleId) {
        Long id= new Long(bundleId);
        Object service= m_services.get(id);
        if (service == null)
            return false;
        long count= decrUseCount(bundleId);
        if (count <= 0) {
            if (m_service instanceof ServiceFactory) {
                Bundle bundle= m_manager.getBundle(bundleId);
                ((ServiceFactory)m_service).ungetService(bundle, m_registration, service);
            }
            m_services.remove(id);
        }
        return true;
    }


    synchronized public void unregisterService() {
        if (m_service instanceof ServiceFactory) {
            for (Iterator i= m_services.entrySet().iterator(); i.hasNext();) {
                Map.Entry entry= (Map.Entry)i.next();
                Long id= (Long)entry.getKey();
                long bundleId= id.longValue();
                Object service= entry.getValue();
                STBundle stBundle= m_manager.getBundle(bundleId);
                if (stBundle == null)
                    continue;
                ((ServiceFactory)m_service).ungetService(stBundle, m_registration, service);
            }
        }
        m_counts.clear();
        m_services.clear();
    }

    public BundleManager getBundleManager() {
        return m_manager;
    }
    public String[] getClasses() {
        return (String[])m_clazzes.clone();
    }
    public Dictionary getProperties() {
        return m_properties;
    }

    synchronized public void clearUseCount(long bundleId) {
    	Long id= new Long(bundleId);
    	m_counts.remove(id);
    }
    /**
     * @return the new count
     */
    public synchronized long incrUseCount(long bundleId) {
    	Long id= new Long(bundleId);
    	Long count= (Long)m_counts.get(id);
        long value= (count == null) ? 1 : count.longValue() + 1;
        m_counts.put(id, new Long(value));
        return value;
    }
    /**
     * @return the new count
     */
    public synchronized long decrUseCount(long bundleId) {
    	Long id= new Long(bundleId);
    	Long count= (Long)m_counts.get(id);
        if (count == null)
            return 0;
        long value= count.longValue();
        if (value <= 1) {
            m_counts.remove(id);
            return 0;
        }
        m_counts.put(id, new Long(--value));
        return value;
    }
    public synchronized long getUseCount(long bundleId) {
    	Long id= new Long(bundleId);
    	Long count= (Long)m_counts.get(id);
        if (count == null)
            return 0;
        long value= count.longValue();
        return value;
    }

  /**
   * Implementation of the ServiceReference.getProperty method.
   * @see ServiceReference.getProperty
   */
    public Object getProperty(String key) {
        return m_properties.get(key);
    }
  /**
   * Implementation of the ServiceReference.getPropertyKeys method.
   * @see ServiceReference.getPropertyKeys
   */
    public String[] getPropertyKeys() {
        ArrayList list= new ArrayList();
        for (Enumeration e= m_properties.keys(); e.hasMoreElements();)
            list.add(e.nextElement());
        String[] keys= new String[list.size()];
        list.toArray(keys);
        return keys;
    }
  /**
   * Implementation of the ServiceReference.getBundle method.
   * @see ServiceReference.getBundle
   */
    public Bundle getBundle() {
        /*
        This method will always return null when the service has been
        unregistered. This can be used to determine if the service has been
        unregistered.
        */
        if (!m_manager.isRegisteredService(this))
            return null;

        return m_bundle;
    }

    public boolean equals(Object obj) {
        if (obj instanceof ServiceReference)
            return m_registration.equals(((STServiceReference)obj).m_registration);
        return super.equals(obj);
    }
    public int hashCode() {
        return m_registration.hashCode();
    }
}
