/*
   AsyncLogWriter.java

   Copyright (C) 2000 by ted stockwell
   All Rights Reserved
*/
package org.servicetango.log;

import org.osgi.service.log.*;

import java.io.*;
import java.text.*;
import java.util.*;

/**
*   A log writer that uses a background thread to write to a log
*
* @author   ted stockwell
*/
public class AsyncLogWriter
implements ILogWriter
{
    public AsyncLogWriter(ILogWriter writer)
    {
        m_writer= writer;
    }

    /**
    * Write the message to the log file
    */
    public void write(String msg)
    {
        /*
        Put the message on the log queue and notify the "logger".
        */
        synchronized (m_queue)
        {
            m_queue.add(msg);

            /*
            start a logger thread to write the message
            */
            if (m_logger == null)
            {
                m_logger= new Logger();
                m_logger.start();
            }
        }
    }

    /**
    *   This thread class writes all the messages in the message queue
    */
    private class Logger extends Thread
    {
        Logger()
        {
        }
        public void run()
        {
            for (;;)
            {
                String msg;
                synchronized (m_queue)
                {
                    /*
                    If there are messages in the queue then write them.
                    */
                    if (m_queue.size() <= 0)
                    {
                        m_logger= null;
                        m_queue.notifyAll();
                        break;
                    }
                    msg= (String)m_queue.remove(0);
                }
                m_writer.write(msg);
            }
        }
    }

    public void clear()
    {
        m_writer.clear();
    }

    /**
    *   Doesn't return until all log messages have been written or a
    *   failure occurs.
    */
    public void flush()
    {
        for (;;) {
            synchronized (m_queue)
            {
                if (m_queue.size() <= 0)
                    break;
                if (m_logger != null) {
                    try {
                        m_queue.wait();
                    }
                    catch (InterruptedException e) {
                    }
                }
            }
        }
    }


    private List m_queue= new ArrayList();
    private Logger m_logger= null;
    private ILogWriter m_writer;
}
