/*
   FileLogWriter.java

   Copyright (C) 1999 by servicetango.org
   All Rights Reserved
*/
package org.servicetango.log;

import java.io.*;
import java.text.*;
import java.util.*;

/**
*   A log writer that writes message to a log file
*
* @author   ted stockwell
*/
public class FileLogWriter
implements ILogWriter
{
    /** file to log() messages to */
    protected String logFileName = null;

    public FileLogWriter(String fileName)
    {
        logFileName= fileName;
    }

    /**
    * Write the message to the log file
    */
    public synchronized void write(String msg)
    {
        if (logFileName != null)
        {
            FileWriter wr = null;
            try
            {
                DateFormat df = new SimpleDateFormat("[dd/MMM/yyyy:HH:mm:ss]");
                df.setTimeZone(TimeZone.getDefault());

                wr = new FileWriter(logFileName, true /*append*/);
                String entry=
                    df.format(new Date())
                    + " "
                    + msg
                    + System.getProperty("line.separator");
                wr.write(entry);
                wr.flush();
            }
            catch (IOException ex)
            {
                // don't do anything
            }
            finally
            {
                if (wr != null)
                {
                    try
                    {
                        wr.close();
                    }
                    catch (IOException ex)
                    {
                    }
                }
            }
        }
    }

    public synchronized void clear()
    {
        if (logFileName != null)
        {
            /*
            To clear the log file I just open and close the file in an
            attempt to create a file of zero length
            */
            try
            {
                FileOutputStream in= new FileOutputStream(logFileName);
                try
                {
                    in.write(new byte[0]);
                    in.flush();
                }
                finally
                {
                    try
                    {
                        in.close();
                    }
                    catch (Exception e)
                    {
                    }
                }
            }
            catch (Exception e)
            {
            }
        }
    }

    /**
    *   Doesn't return until all log messages have been written or a
    *   failure occurs.
    *   This method needn't do naything since output is flushed on
    *   every write.
    */
    public void flush()
    {
    }
}
