/*
   ILogWriter.java

   Copyright (c) 2000 by ted stockwell
   All Rights Reserved
*/
package org.servicetango.log;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 *  An interface for writing to a Log.
 *  This interface is used to implement a "strategy" pattern for the Log
 *  class that actually writes to some physical log.
 *
 * @author   ted stockwell
 */
public interface ILogWriter
{
    /**
    *   Writes the given msg to the log.
    *   The msg may be reformatted or extra text may be prepended or
    *   appended to the original message before writing to the log.
    *   The text that is actually written to the log is returned.
    */
    public void write(String msg);

    /**
    *   Clear the log file
    */
    public void clear();

    /**
    *   Doesn't return until all log messages have been written or a
    *   failure occurs.
    */
    public void flush();
}

