/*
   OutputStreamLogWriter.java

   Copyright (c) 2000 by ted stockwell
   All Rights Reserved
*/
package org.servicetango.log;

import java.io.*;
import java.text.*;
import java.util.*;

/**
*   A log writer that writes messages to an output stream
*
* @author   ted stockwell
*/
public class OutputStreamLogWriter
implements ILogWriter
{
    protected OutputStream m_stream = null;
    protected Writer m_writer = null;


    public OutputStreamLogWriter(OutputStream stream) {
        m_stream= stream;
        if (m_stream != null)
            m_writer= new OutputStreamWriter(m_stream);
    }

    /**
    * Write the message to the log file
    */
    public synchronized void write(String msg)
    {
        if (m_writer != null) {
            try {
                DateFormat df = new SimpleDateFormat("[dd/MMM/yyyy:HH:mm:ss]");
                df.setTimeZone(TimeZone.getDefault());

                String entry=
                    df.format(new Date())
                    + " "
                    + msg
                    + System.getProperty("line.separator");
                m_writer.write(entry);
                m_writer.flush();
            }
            catch (IOException ex) {
                // don't do anything
            }
        }
    }

    public synchronized void clear()
    {
    }

    /**
    *   Doesn't return until all log messages have been written or a
    *   failure occurs.
    *   This method needn't do naything since output is flushed on
    *   every write.
    */
    public void flush()
    {
    }
}
