package org.servicetango.service.http;

import java.net.*;
import java.util.*;
import java.io.*;
import org.osgi.framework.*;
import org.apache.tomcat.logging.*;


public class STHttpActivator
implements BundleActivator, STHttpConstants
{
    static public final int DEBUG_LEVEL= 0;

    public void start(BundleContext bundlecontext)
    throws BundleException
    {
        hsmap = new Hashtable();

        /*
        set the protocol
        */
        String _protocol= bundlecontext.getProperty(KEY_PROTOCOL);
        if (_protocol == null  ||  _protocol.length() <= 0)
            _protocol= System.getProperty(KEY_PROTOCOL);
        if (_protocol == null  ||  _protocol.length() <= 0)
            _protocol= DEFAULT_PROTOCOL;

        /*
        set the host name
        */
        String hostname= bundlecontext.getProperty(KEY_HOSTNAME);
        if (hostname == null  ||  hostname.length() <= 0)
            hostname= System.getProperty(KEY_HOSTNAME);
        if (hostname == null  ||  hostname.length() <= 0) {
            try {
                hostname= InetAddress.getLocalHost().getHostName();
            }
            catch (UnknownHostException x) { }
        }
        InetAddress _inetaddress= null;
        try {
            if(hostname != null)
                _inetaddress = InetAddress.getByName(hostname);
        }
        catch(UnknownHostException unknownhostexception) {
            throw new BundleException("Could not start HttpService", unknownhostexception);
        }

        /*
        set the port
        */
        String sPort= bundlecontext.getProperty(KEY_PORT);
        if (sPort == null  ||  sPort.length() <= 0)
            sPort= System.getProperty(KEY_PORT);
        int _port= DEFAULT_PORT;
        if(sPort != null) {
            try {
                _port= Integer.parseInt(sPort);
            }
            catch(NumberFormatException numberformatexception) {
                throw new BundleException("Could not start HttpService; invalid port.", numberformatexception);
            }
        }

        /*
        register a service factory
        */
        Hashtable hashtable = new Hashtable();
        hashtable.put("Description", "The OSGI HTTP Service");
        final InetAddress inetaddress= _inetaddress;
        final int port = _port;
        final String protocol= _protocol;
        ServiceFactory servicefactory = new ServiceFactory() {
            public Object getService(Bundle bundle, ServiceRegistration serviceregistration) {
                STHttpServiceImpl httpserviceimpl = new STHttpServiceImpl(m_server, protocol, inetaddress, port, bundle);
                synchronized(hsmap) {
                    if(hsmap.contains(bundle))
                        throw new IllegalStateException("HttpServiceImpl already exists for bundle " + bundle);
                    hsmap.put(bundle, httpserviceimpl);
                }
                return httpserviceimpl;
            }

            public void ungetService(Bundle bundle, ServiceRegistration serviceregistration, Object obj) {
            }

        };
        bundlecontext.registerService("org.osgi.service.http.HttpService", servicefactory, hashtable);
        bundlecontext.addBundleListener(
            new BundleListener() {
                public void bundleChanged(BundleEvent bundleevent) {
                    if(bundleevent.getType() == 4) {
                        STHttpServiceImpl httpserviceimpl = (STHttpServiceImpl)hsmap.remove(bundleevent.getBundle());
                        if(httpserviceimpl != null)
                            httpserviceimpl.unregisterAll();
                    }
                }
            }
        );

        /*
        TODO
        UNFINISHED

        Install Tomcat log interfaces.
        By default Tomcat sends log messages to standard output.
        To make this stop happening you must install your own log interface.

        Logging.putLogger(...);
        */


        ///////////////////////////////////////////
        //
        //  start the server
        //
        m_server = new STServer();

        m_server.setDebug(DEBUG_LEVEL);

        /*
        set the working directory

        ask the bundle context for a temporary file and use the absolute path of
        the temporary file to create a working directory
        */
        File dataFile= bundlecontext.getDataFile("DataFile");
        String parent= dataFile.getParent();
        File workdir= new File(parent+"/workdir");
        workdir.mkdirs();
        m_server.setWorkDir(workdir.getAbsolutePath());

        /*
        add an endpoint
        the working directory must be set before an endpoint is created
        */
        m_server.addEndpoint(port, inetaddress, hostname);

        /*
        now start the server
        */
        m_server.start();
    }

    public void stop(BundleContext bundlecontext)
    throws BundleException
    {
        STServer hs= m_server;
        m_server = null;
        hsmap = null;

        try {
            hs.stop();
        }
        catch (Throwable t) { }
    }

    public STHttpActivator() { }

    private STServer m_server;
    private Hashtable hsmap;
}
