package org.servicetango.service.http;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.osgi.framework.*;
import org.osgi.service.http.*;
import org.apache.tomcat.core.*;
import org.servicetango.util.*;

class STHttpServiceImpl
implements HttpService
{

    STHttpServiceImpl(STServer httpserver, String protocol, InetAddress inetaddress, int port, Bundle bundle1) {
        server = httpserver;
        String host= "*";
        if(inetaddress != null) {
            String s1 = inetaddress.getHostName();
            host= s1 != null ? s1 : "*";
        }
        try {
            curl = new URL(protocol, host, port, "/");
        }
        catch(MalformedURLException _ex) {
            throw new InternalError();
        }
        bundle = bundle1;
        contextMaps = new Hashtable();
    }

    public void registerServlet(String alias, Servlet servlet, Dictionary initParams, HttpContext httpcontext)
    throws ServletException, NamespaceException
    {
        if (initParams == null)
          initParams= CollectionUtils.toDictionary(new HashMap(0));
        URL url = getURL(alias);
        if (httpcontext == null)
            throw new NullPointerException("An HttpContext must be supplied");
        server.registerServlet(alias, servlet, initParams, httpcontext);
    }

    public void registerResources(String alias, String name, HttpContext httpcontext)
    throws NamespaceException
    {
        URL url = getURL(alias);
        if(name.endsWith("/"))
            throw new NamespaceException("illegal resource name; name cannot end with a '/'");
        if(httpcontext == null)
            throw new NamespaceException("An HttpContext must be supplied");
        try {
            server.registerResources(alias, name, httpcontext);
        }
        catch (Exception x) {
            throw new NamespaceException("Could not register servlet", x);
        }
    }

    public void unregister(String alias) {
        try {
            server.unregister(alias);
        }
        catch (Exception x) {
            x.printStackTrace();
        }
    }
    public void unregisterAll() {
        try {
            server.unregisterAll();
        }
        catch (Exception x) {
            x.printStackTrace();
        }
    }

    private URL getURL(String s)
    {
        if((s.startsWith("/") || s.startsWith("http://")) && !s.endsWith("/"))
            try
            {
                return new URL(curl, s);
            }
            catch(MalformedURLException _ex) { }
        throw new IllegalArgumentException("illegal alias name");
    }


    private static InetAddress getAnyInetAddress()
    {
        try
        {
            return InetAddress.getByName("0.0.0.0");
        }
        catch(UnknownHostException _ex)
        {
            throw new InternalError();
        }
    }

    private InetAddress getInetAddress(URL url)
    {
        String s = url.getHost();
        if(s.equals("*"))
            return anyInetAddress;
        try
        {
            return InetAddress.getByName(s);
        }
        catch(UnknownHostException _ex)
        {
            throw new IllegalArgumentException("illegal host name:" + s);
        }
    }


    private STServer server;
    private URL curl;
    private Bundle bundle;
    private Hashtable contextMaps;
    private static final InetAddress anyInetAddress = getAnyInetAddress();

}
