package org.servicetango.service.http;

import java.util.*;
import java.net.*;
import java.io.*;
import java.security.*;
import javax.servlet.*;

import org.osgi.service.http.*;
import org.osgi.framework.*;

import org.apache.tomcat.startup.*;
import org.apache.tomcat.core.*;
import org.apache.tomcat.request.*;
import org.apache.tomcat.service.*;
import org.apache.tomcat.service.http.*;
import org.apache.tomcat.session.StandardSessionInterceptor;
import org.apache.tomcat.context.*;

import org.servicetango.core.*;

public class STServer
{
    ContextManager contextM= new ContextManager();

    /** Right now we assume all web apps use the same
	servlet API version. This will change after we
	finish the FacadeManager implementation
    */
    FacadeManager facadeM=null;

    // configurable properties
    int debug=0;

    public STServer() {
    }

    // -------------------- Properties - set before start

    /** Set debugging - must be called before anything else
     */
    public void setDebug( int debug ) {
	this.debug=debug;
    }

    public void setWorkDir( String dir ) {
        contextM.setWorkDir(dir);
    }

    // -------------------- Endpoints --------------------

    /** Add a web service on the specified address. You must add all the
     *  endpoints before calling start().
     */
    public void addEndpoint( int port, InetAddress addr , String hostname) {
	if(debug>0)
            log( "addConnector " + port + " " + addr + " " + hostname );

	PoolTcpConnector sc=new PoolTcpConnector();
	sc.setServer( contextM );
	sc.setDebug(debug);
	sc.setAttribute( "vhost_port" , new Integer( port ) );
	if( addr != null ) sc.setAttribute( "vhost_address", addr );
	if( hostname != null ) sc.setAttribute( "vhost_name", hostname );

	sc.setTcpConnectionHandler( new HttpConnectionHandler());

	contextM.addServerConnector(  sc );
    }

    /** Add a secure web service.
     */
    public void addSecureEndpoint( int port, InetAddress addr, String hostname,
				    String keyFile, String keyPass )
    {
	if(debug>0)
            log( "addSecureConnector " + port + " " + addr + " " + hostname );

	PoolTcpConnector sc=new PoolTcpConnector();
	sc.setServer( contextM );
	contextM.setSecurePort( port );
	sc.setAttribute( "vhost_port" , new Integer( port ) );
	if( addr != null ) sc.setAttribute( "vhost_address", addr );
	if( hostname != null ) sc.setAttribute( "vhost_name", hostname );

	sc.setAttribute( "socketFactory",
			 "org.apache.tomcat.net.SSLSocketFactory");
	//	System.out.println("XXX " + keyFile + " " + keyPass);
	HttpConnectionHandler ch=new HttpConnectionHandler();
	ch.setSecure(true);
	sc.setTcpConnectionHandler( ch );
	// XXX add the secure socket

	contextM.addServerConnector(  sc );
    }

    // -------------------- Context add/remove --------------------
    synchronized public void registerServlet(String alias, Servlet servlet, Dictionary initParams, HttpContext httpcontext)
    throws ServletException, NamespaceException
    {
        try {
            if(debug>0)
                log( "add context \"" + alias);

            /*
            if the given alias is already registered then throw a NamespaceException
            */
            for (Enumeration e= contextM.getContexts(); e.hasMoreElements();) {
              STContext ctx= (STContext)e.nextElement();
              if (ctx.getPath().equals(alias))
                throw new NamespaceException("Alias already exists:'"+alias+"'");
            }

            STServletWrapper wrapper= new STServletWrapper(servlet);
            STContext ctx=new STContext(contextM, alias, initParams, httpcontext);
            wrapper.setContext(ctx);
            ctx.addServlet(wrapper);
            ctx.setDebug(debug);
            ctx.setContextManager(contextM);
            ctx.setPath("");
            ctx.getContainer();
            contextM.addContext(ctx);

            Container map=new Container();
            map.setContext(ctx);
            map.setHandler(wrapper);
            map.setPath(alias);
            contextM.addContainer(map);

            initContext(ctx);
        }
        catch (TomcatException x) {
            throw new STServletException("Could not register servlet", x);
        }
    }
    synchronized public void registerResources(String alias, String name, HttpContext httpcontext)
    throws TomcatException
    {
	if(debug>0)
          log( "add context \"" + alias + "\" " + name);

        STContext ctx=new STContext(contextM, alias, name, httpcontext);
        ctx.setDebug(debug);
        ctx.setContextManager(contextM);
        ctx.setPath("");
        contextM.addContext(ctx);

        Container map=new Container();
        map.setContext(ctx);
        //??map.setHandler(wrapper);
        map.setPath(alias);
        contextM.addContainer(map);

        initContext(ctx);
    }
    public void unregister(String alias)
    throws TomcatException
    {
	if(debug>0)
            log( "remove context " + alias);
        for (Enumeration e= contextM.getContexts(); e.hasMoreElements();) {
          STContext ctx= (STContext)e.nextElement();
          if (ctx.getPath().equals(alias)) {
            Container map= ctx.getContainer();
            contextM.removeContext(ctx);
            contextM.removeContainer(map);
            break;
          }
        }
    }
    public void unregisterAll()
    throws TomcatException
    {
	if(debug>0)
            log( "remove all contexts ");
        for (Enumeration e= contextM.getContexts(); e.hasMoreElements();) {
          STContext ctx= (STContext)e.nextElement();
          contextM.removeContext(ctx);
        }
    }

    Hashtable extraClassPaths=new Hashtable();

    /** The application may want to add an application-specific path
	to the context.
    */
    public void addClassPath( ServletContext context, String cpath ) {
	if(debug>0)
            log( "addClassPath " + context.getRealPath("") + " " + cpath );

	try {
	    Vector cp=(Vector)extraClassPaths.get(context);
	    if( cp == null ) {
		cp=new Vector();
		extraClassPaths.put( context, cp );
	    }
	    cp.addElement( cpath );
	} catch( Exception ex ) {
	    ex.printStackTrace();
	}

	// XXX This functionality can be achieved by setting it in the parent
	// class loader ( i.e. the loader that is used to load tomcat ).

	// It shouldn't be needed if the web app is self-contained,
    }


    /** This will make the context available.
     */
    void initContext(Context ctx) {
	try {
	    contextM.initContext( ctx );

	    ServletLoader sl=ctx.getServletLoader();
	    //	    System.out.println("ServletLoader: " + sl );
	    Object pd=ctx.getProtectionDomain();
	    //	    System.out.println("Ctx.pd " + pd);

	    // Add any extra cpaths
	    Vector cp=(Vector)extraClassPaths.get(ctx);
	    if( cp!=null ) {
		for( int i=0; i<cp.size(); i++ ) {
		    String cpath=(String)cp.elementAt(i);
		    sl.addRepository( new File(cpath), pd);
		}
	    }
	} catch( Exception ex ) {
	    ex.printStackTrace();
	}
    }

    // -------------------- Start/stop

    public void start()
    throws BundleException
    {
        try {
            contextM.setDefaults();
            contextM.init();
            contextM.start();
            if(debug>0)
                log( "Started" );
       }
       catch(Exception x) {
        throw new STException("Could not start HTTP service", x);
       }
    }

    public void stop()
    throws Exception
    {
      contextM.stop();
    }




    // -------------------- Utils --------------------
    private void log( String s ) {
	System.out.println("org.servicetango.service.http: " + s );
    }

}
