package org.servicetango.service.log;

import java.util.*;

import org.osgi.framework.*;
import org.osgi.service.log.*;


public class Log {

    Log(int i, LogListeners loglisteners) {
        logEnd = 0;
        if(i <= 0) {
            throw new IllegalArgumentException();
        }
        else {
            logSize = i + 1;
            theLog = new LogEntry[logSize];
            logEnd = 0;
            theLog[logEnd] = logEndMarker;
            theLogListeners = loglisteners;
            logSeverityThreshold = 4;
            return;
        }
    }

    public void add(Bundle bundle, ServiceReference servicereference, int i, String s, Throwable throwable) {
        if(i < 1 || i > 4)
            throw new IllegalArgumentException();
        if(i <= logSeverityThreshold) {
            LogEntryImpl logentryimpl = new LogEntryImpl(bundle, servicereference, i, s, throwable);
            synchronized(this) {
                theLog[logEnd] = logentryimpl;
                logEnd = (logEnd + 1) % logSize;
                theLog[logEnd] = logEndMarker;
            }
            theLogListeners.notifyListeners(logentryimpl);
        }
    }

    public synchronized Enumeration getLog() {
        return new Enumeration() {
            public synchronized boolean hasMoreElements() {
                return current != Log.logEndMarker;
            }

            public synchronized Object nextElement() {
                LogEntry logentry = current;
                if(current == Log.logEndMarker)
                    throw new NoSuchElementException("Log enumeration");
                synchronized(Log.this) {
                    if(current == theLog[index]) {
                        index = ((index - 1) + logSize) % logSize;
                        current = theLog[index];
                    }
                    else
                        current = Log.logEndMarker;
                }
                return logentry;
            }

            private int index;
            private LogEntry current;
            {
                index = ((logEnd - 1) + logSize) % logSize;
                current = theLog[index];
            }
        };
    }

    public int getLogSize() {
        return logSize;
    }

    public synchronized void setLogSize(int i) {
        if(i <= 0)
            throw new IllegalArgumentException();
        if(i == logSize - 1)
            return;
        int j = i > logSize - 1 ? logSize - 1 : i;
        LogEntry alogentry[] = new LogEntry[i + 1];
        for(; j >= 0; j--)
        {
            alogentry[j] = theLog[logEnd];
            logEnd = ((logEnd - 1) + logSize) % logSize;
        }

        theLog = alogentry;
        logSize = i + 1;
        logEnd = i;
    }

    public int getSeverityThreshold() {
        return logSeverityThreshold;
    }

    public void setSeverityThreshold(int i) {
        if(i < 1 || i > 4) {
            throw new IllegalArgumentException();
        }
        else {
            logSeverityThreshold = i;
            return;
        }
    }

    private LogEntry theLog[];
    private int logSize;
    private int logEnd;
    private LogListeners theLogListeners;
    private static final int LOG_MIN_LEVEL = 1;
    private static final int LOG_MAX_LEVEL = 4;
    private int logSeverityThreshold;
    private static final LogEntry logEndMarker = null;
}
