/*
   CaselessProfile.java
*/

package org.servicetango.util;
import java.util.*;


/**
 * A profile map that is insensitive to the case of keys.
 *
 * @author ted stockwell [emorning@mediaone.net]
 */
public class CaselessProfile extends ProfileMap
{
    /**
     * Save the original key with a key name equal to the upper case key name.
     */
    protected HashMap m_names= new HashMap();

    public CaselessProfile(Map map) {
	    super(map);
    }
    public CaselessProfile() {
	    super();
    }
    public CaselessProfile(int size) {
	    super(size);
    }

    public boolean containsKey(Object key)
    {
        boolean value= super.containsKey(key);
        if (!value)
            if ((key= m_names.get(key.toString().toUpperCase())) != null)
                value= super.containsKey(key);
        return value;
    }

    public Object get(Object key)
    {
        Object value= super.get(key);
        if (value == null)
            if ((key= m_names.get(key.toString().toUpperCase())) != null)
                value= super.get(key);
        return value;
    }

    public Object put(Object key, Object value)
    {
        Object o= super.put(key, value);
        m_names.put(key.toString().toUpperCase(), key);
        return o;
    }

    public Object remove(Object key)
    {
        Object value= super.remove(key);
        if (value == null)
            if ((key= m_names.get(key.toString().toUpperCase())) != null)
                value= super.remove(key);
        return value;
    }

    public void putAll(Map t)
    {
        for (Iterator i= t.entrySet().iterator(); i.hasNext();)
        {
            Map.Entry entry= (Map.Entry)i.next();
            put(entry.getKey(), entry.getValue());
        }
    }
    public void clear()
    {
        super.clear();
        m_names.clear();
    }
}
