/*
   CollectionUtils.java

   Copyright (c) 2000 by ted stockwell
   All Rights Reserved
*/

package org.servicetango.util;
import java.util.*;
import java.io.*;
import java.sql.Clob;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;
import java.sql.Types;



/**
 * Miscellaneouse Collection utilites
 *
 * @author ted stockwell [emorning@mediaone.net]
 */
public class CollectionUtils
{
    /**
     * Create a Dictionary object from a Map
     */
    static public Dictionary toDictionary(final Map map)
    {
        return new Dictionary() {
            public int size() { return map.size(); }
            public boolean isEmpty() { return map.isEmpty(); }
            public Enumeration keys() { return Collections.enumeration(map.keySet()); }
            public Enumeration elements() { return Collections.enumeration(map.values()); }
            public Object get(Object key) { return map.get(key); }
            public Object put(Object key, Object value) { return map.put(key, value); }
            public Object remove(Object key) { return map.remove(key); }
        };
    }


    /**
     * Create a Properties object from a Map
     */
    static public Properties toProperties(Map map)
    {
        Properties properties= new Properties();
        addAll(properties, map);
        return properties;
    }

    /**
     * Convert a Map to a Hashtable
     */
    static public Map toMap(Dictionary table)
    {
        Map map= new HashMap(table.size());
        putAll(map, table);
        return map;
    }

    /**
     * Create a Properties object from a Map
     */
    static public void putAll(Map map, Dictionary table)
    {
        for (Enumeration e= table.keys(); e.hasMoreElements();)
        {
            Object key= e.nextElement();
            map.put(key, table.get(key));
        }
    }

    /**
     * Create a Vector from a Collection
     */
    static public Vector toVector(Collection collection) {
        if (collection instanceof Vector)
            return (Vector)collection;
        Vector v= new Vector(collection.size());
        for (Iterator i= collection.iterator(); i.hasNext();)
            v.addElement(i.next());
        return v;
    }

    /**
     * Create a Vector from an array of objects
     */
    static public Vector toVector(Object[] values) {
        Vector v= new Vector(values.length);
        for (int i= 0; i < values.length;)
            v.addElement(values[i++]);
        return v;
    }

    /**
     * Create a List from a Collection
     */
    static public List toList(Collection collection) {
        if (collection instanceof List)
            return (List)collection;
        return new ArrayList(collection);
    }

    /**
     * Add all the values in a Map to a Properties object.
     */
    static public void addAll(Properties properties, Map map)
    {
        for (Iterator i= map.entrySet().iterator(); i.hasNext();)
        {
            Map.Entry entry= (Map.Entry)i.next();
            Object key= entry.getKey();
            Object value= entry.getValue();
            properties.put(key.toString(), value.toString());
        }
    }
    /**
     *  Convert the current record in a RecordSet to a Map of values.
     */
    static final public Map toMap(ResultSet resultSet)
    throws SQLException
    {
        return toMap(resultSet, null);
    }

    /**
     *  Convert the current record in a RecordSet to a Map of values.
     *
     *  @param  columnToKey  Maps column names in the given record set
     *          to keys names in the resulting Map.  May be null, in
     *          which case the keys in the resulting map have the same
     *          value as the column names.
     *
     */
    static public Map toMap(ResultSet resultSet, Map columnToKey)
    throws SQLException
    {
        ResultSetMetaData metaData= resultSet.getMetaData();
        int columnCount= metaData.getColumnCount();
        Map map= new HashMap(columnCount);
        for (int iColumn= 1; iColumn <= columnCount; iColumn++) {
            String key= metaData.getColumnName(iColumn);
            if (columnToKey != null) {
                String c2k= (String)columnToKey.get(key);
                if (c2k != null)
                    key= c2k;
            }
            Object value= resultSet.getObject(iColumn);
            map.put(key, value);
        }
        return map;
    }

    /**
     *  Convert the current record in a RecordSet to a Properties
     *  object (where all the values are strings).
     *
     *  @param  columnToKey  Maps column names in the given record set
     *          to keys names in the resulting Map.  May be null, in
     *          which case the keys in the resulting map have the same
     *          value as the column names.
     *
     */
    static public Map toProperties(ResultSet resultSet, Map columnToKey)
    throws SQLException
    {
        ResultSetMetaData metaData= resultSet.getMetaData();
        int columnCount= metaData.getColumnCount();
        Map map= new HashMap(columnCount);
        for (int iColumn= 1; iColumn <= columnCount; iColumn++) {
            String columnName= metaData.getColumnName(iColumn);
            String key= columnName;
            if (columnToKey != null) {
                String c2k= (String)columnToKey.get(key);
                if (c2k != null)
                    key= c2k;
            }
            try {
                Object value= null;
                /*
                if (metaData.getColumnType(iColumn) == Types.LONGVARCHAR) {
                    Clob clob= resultSet..getClob(iColumn);
                    //value= clob.toString();
                } else
                */
                    value= resultSet.getString(iColumn);
                map.put(key, value);
            }
            catch (SQLException x) {
                System.out.println("iColumn="+iColumn);
                System.out.println("columnName="+columnName);
                System.out.println("columnType="+metaData.getColumnType(iColumn));
                System.out.println("columnTypeName="+metaData.getColumnTypeName(iColumn));
                x.printStackTrace();
                throw x;
            }
        }
        return map;
    }
    static public Map toProperties(ResultSet resultSet)
    throws SQLException
    {
        return toProperties(resultSet, null);
    }

    /**
    *   Takes the given map and returns a map that reverses the mapping.
    */
    static public Map reverseMap(Map map) {
        Map reverse= new HashMap(map.size());
        for (Iterator i= map.entrySet().iterator(); i.hasNext();) {
            Map.Entry entry= (Map.Entry)i.next();
            reverse.put(entry.getValue(), entry.getKey());
        }
        return reverse;
    }


    /**
    Applies a mapping to another map.

    The given data map is transformed into the returned map by
    including in the returned map only those items in the data map that
    have keys listed in the keyMap map.  The keys in returned map are
    equal to the values of the associated key in the keyMap Map.  Got that?
    */
    static public Map transform(Map keyMap, Map data) {
        Map transform= new HashMap(keyMap.size());
        for (Iterator i= keyMap.entrySet().iterator(); i.hasNext();) {
            Map.Entry entry= (Map.Entry)i.next();
            Object key= entry.getKey();
            Object newKey= entry.getValue();
            Object value= data.get(key);
            if (value != null)
                transform.put(newKey, value);
        }
        return transform;
    }


    /**
    *   Returns a Map value as a String
    */
	final public String getString(Map map, Object key)	{
        Object o= map.get(key);
        if (o instanceof String)
            return (String)o;
        return o.toString();
    }

    /**
    *   Convenience method that returns a value as a date.
    */
	static public final Date getDate(Map map, String key)	{
        return LangUtils.toDate(map.get(key));
    }

    /**
    *   Convenience method that returns a value as an int
    */
	static public final int getInt(Map map, String key, int defaultValue)	{
        return LangUtils.toInt(map.get(key), defaultValue);
	}

    /**
    *   Convenience method that returns a value as a byte
    */
	static final public byte getByte(Map map, String key, byte defaultValue)	{
        return LangUtils.toByte(map.get(key), defaultValue);
	}

    /**
    *   Convenience method that returns a value as a boolean
    */
	public boolean getBool(Map map, String key, boolean defaultValue)	{
        return LangUtils.toBoolean(map.get(key), defaultValue);
	}
}
