/*
   FilterMap.java

   Copyright (c) 2000 by ted stockwell
   All Rights Reserved
*/

package org.servicetango.util;
import java.util.*;

/**
Filter pattern class for Maps

@author  ted stockwell [emorning@mediaone.net]
*/
public class FilterMap
implements Map, java.io.Serializable
{
	protected Map m_map;

	public FilterMap(Map map) {
		m_map= map;
	}
        public FilterMap(int count) {
    	    this(new HashMap(count));
        }
        public FilterMap() {
    	    this(new HashMap());
        }


    /**
    Returns a reference to the FilterMap's underlying Map
    */
    protected Map getMap() {
        return m_map;
    }
    /**
    Chang the underlying Map used by this FilterMap
    */
    protected void setMap(Map value) {
        m_map= value;
    }

    public int size()
	{
		return m_map.size();
	}
    public boolean isEmpty()
	{
		return m_map.isEmpty();
	}
    public boolean containsKey(Object key)
	{
		return m_map.containsKey(key);
	}
    public boolean containsValue(Object value)
	{
		return m_map.containsValue(value);
	}
    public Object get(Object key)
	{
		return m_map.get(key);
	}
    public Object put(Object key, Object value)
	{
		return m_map.put(key, value);
	}
    public Object remove(Object key)
	{
		return m_map.remove(key);
	}
    public void putAll(Map t)
	{
		m_map.putAll(t);
	}
    public void clear()
	{
		m_map.clear();
	}
    public Set keySet()
	{
		return m_map.keySet();
	}
    public Collection values()
	{
		return m_map.values();
	}
    public Set entrySet()
	{
		return m_map.entrySet();
	}
    public int hashCode()
	{
		return m_map.hashCode();
	}
    public String toString()
	{
		return m_map.toString();
	}

    private Collection equals_object_list= null;
    /**
     * Fixes a problem in the AbstractMap.equals method that causes the method
     * to generate a stack overflow when a Map recursively contains itself.
     */
    public synchronized boolean equals(Object o) {
    	if (o == this)
    	    return true;

    	if (!(o instanceof Map))
    	    return false;

    	Map t = (Map) o;
    	if (t.size() != size())
    	    return false;

        /*
        If, during the process of comparing all the elements in this map, we
        have recursively called into this method, and we are being asked to
        compare an object that has already been checked then just return true.
        */
        boolean clear_equals_object_list= false;
        if (equals_object_list == null) {
            clear_equals_object_list= true;
            equals_object_list= new HashSet();
        }
        if (equals_object_list.contains(o))
            return true;
        equals_object_list.add(o);

        boolean isEqual= true;
    	Iterator i = entrySet().iterator();
    	while (i.hasNext()) {
    	    Map.Entry e = (Map.Entry) i.next();
    	    Object key = e.getKey();
    	    Object value = e.getValue();

    	    if (value == null) {
        	if (!(t.get(key)==null && t.containsKey(key))) {
                    isEqual= false;
                    break;
                }
    	    } else {
        	if (!value.equals(t.get(key))) {
                    isEqual= false;
                    break;
                }
    	    }
    	}

        if (clear_equals_object_list)
            equals_object_list= null;

    	return isEqual;
    }


    /**
    *   Convenience method that returns a value as a String
    */
    public String getString(String key) {
        return LangUtils.toString(get(key));
    }

    /**
    *   Convenience method that returns a value as a date
    */
	public Date getDate(String key) {
        return LangUtils.toDate(get(key));
	}

    /**
    *   Convenience method that returns a value as an Integer
    */
    public Integer getInteger(String key) {
        return LangUtils.toInteger(get(key));
    }

    /**
    *   Convenience method that returns a value as an int
    */
	public int getInt(String key, int defaultValue)	{
        return LangUtils.toInt(get(key), defaultValue);
	}

    /**
    *   Convenience method that inserts an integer into this map
    */
	public void setInt(String key, int value) {
            put(key, new Integer(value));
	}


	public void setLong(String key, long value) {
            put(key, new Long(value));
	}
	public long getLong(String key, long defaultValue)	{
            return LangUtils.toLong(get(key), defaultValue);
	}

    /**
    *   Convenience method that returns a value as a byte
    */
	public byte getByte(String key, byte defaultValue)	{
        return LangUtils.toByte(get(key), defaultValue);
	}
    /**
    *   Convenience method that inserts a byte into this map
    *   (converts the byte into a string)
    */
	public void setByte(String key, byte value) {
        put(key, new Byte(value));
	}

    /**
    *   Convenience method that returns a value as a boolean
    */
	public boolean getBoolean(String key, boolean defaultValue)	{
        return LangUtils.toBoolean(get(key), defaultValue);
	}
    /**
    *   Convenience method that inserts a boolean into this map
    */
	public void setBoolean(String key, boolean value) {
        put(key, new Boolean(value));
	}
    /**
    *   Convenience method that returns a value as a TimeZone
    */
    public TimeZone getTimeZone(String key)	{
        return LangUtils.toTimeZone(get(key));
    }
}
