/*
   IOUtils.java

   Copyright (c) 2000 by ted stockwell
   All Rights Reserved
*/

package org.servicetango.util;
import java.io.*;



/**
 * Miscellaneouse IO utilites
 *
 * @author ted stockwell [emorning@mediaone.net]
 */
public class IOUtils
{
    /**
     * Remove a directory along with everything in it
     */
    static public void removeAll(String path)
    throws IOException
    {
        removeAll(new File(path));
    }
    /**
     * Remove a directory along with everything in it
     */
    static public void removeAll(File dir)
    throws IOException
    {
        /*
        If the given path doesn;t exist then, hey!, we're done.
        */
        if (!dir.exists())
            return;

        /*
        if the given path is not a directory then just try to delete that one file
        */
        if (!dir.isDirectory()) {
            if (!dir.delete())
                throw new IOException("Failed to delete "+dir);
            return;
        }

        ///////////////////
        // the given path is a directory

        /*
        first, delete all the contents of the directory
        */
        String[] list= dir.list();
        if (list != null) {
            for (int i= 0; i < list.length; i++) {
                File child= new File(dir, list[i]);
                removeAll(child);
            }
        }

        /*
        now, delete the directory itself
        */
        if (!dir.delete())
            throw new IOException("Failed to delete "+dir);
    }

    public static void copyFile(String sourceName, String destName)
    throws IOException
    {
        InputStream is= null;
        OutputStream os= null;
        try {
            is= new FileInputStream(sourceName);
            os= new FileOutputStream(destName);
            BufferedInputStream in= new BufferedInputStream(is);
            BufferedOutputStream out= new BufferedOutputStream(os);
            byte[] buffer= new byte[32*1024];
            for (;;) {
                int count= in.read(buffer);
                if (count == -1)
                    break;
                out.write(buffer, 0, count);
            }
            out.flush();
        }
        finally {
            if (is != null) {
                try { is.close(); } catch (Throwable t) { }
            }
            if (os != null) {
                try { os.close(); } catch (Throwable t) { }
            }
        }
    }
}
