package org.servicetango.util;

import java.lang.reflect.*;
import java.math.*;
import java.util.*;
import org.osgi.framework.*;


public class LDAPUtils {

    public static void parse(String s)
    throws InvalidSyntaxException
    {
        match(null, s);
    }
    public static boolean match(Dictionary dictionary, String s)
    throws InvalidSyntaxException
    {
        if(s == null)
            return true;
        LDAPQuery ldapquery = new LDAPQuery(s);
        char ac[] = s.trim().toCharArray();
        boolean flag = ldapquery.compile(dictionary, ac, 0);
        if(ldapquery.currentIndex != ac.length - 1)
            throw new InvalidSyntaxException("extra characters starting at " + (ldapquery.currentIndex + 1), ldapquery.filter);
        else
            return flag;
    }


    static private class LDAPQuery
    {
        LDAPQuery(String s)
        {
            filter = null;
            filter = s;
        }

        private boolean compile(Dictionary dictionary, char ac[], int i)
        throws InvalidSyntaxException
        {
            int j;
            for(j = i; j < ac.length && Character.isWhitespace(ac[j]); j++);
            if(j >= ac.length)
                throw new InvalidSyntaxException("unexpected end of filter at " + j, filter);
            if(ac[j] != '(')
                throw new InvalidSyntaxException("missing '(' at " + j, filter);
            for(j++; j < ac.length && Character.isWhitespace(ac[j]); j++);
            if(j >= ac.length)
                throw new InvalidSyntaxException("unexpected end of filter at " + j, filter);
            char c = ac[j];
            if(c != '&' && c != '|' && c != '!')
            {
                int k = j;
                int l = -1;
                while(k < ac.length)
                {
                    if(ac[k] == '\\')
                    {
                        k += 2;
                        continue;
                    }
                    if(ac[k] == ')')
                        break;
                    if(ac[k] == '(')
                        throw new InvalidSyntaxException("'(' unexpected at " + k, filter);
                    if(ac[k] == '=')
                        l = k;
                    k++;
                }
                if(k >= ac.length)
                    throw new InvalidSyntaxException("unmatched '(' near " + j, filter);
                if(l == -1)
                    throw new InvalidSyntaxException("no valid filtertype found in the filtercomp starting at " + j, filter);
                currentIndex = k;
                if(dictionary == null)
                    return false;
                char c1 = ac[l - 1];
                char ac1[] = null;
                String s;
                if(c1 != '~' && c1 != '<' && c1 != '>')
                {
                    c1 = ac[l + 1];
                    s = new String(ac, j, l - j);
                    if(c1 != '*' || k - l - 1 > 0)
                        c1 = ac[l];
                    else
                        l++;
                } else
                {
                    s = new String(ac, j, l - j - 1);
                }
                Object obj = dictionary.get(s.toLowerCase());
                boolean flag2;
                if(obj == null)
                    flag2 = false;
                else
                if(c1 == '*')
                {
                    flag2 = true;
                } else
                {
                    ac1 = new char[k - l - 1];
                    System.arraycopy(ac, l + 1, ac1, 0, k - l - 1);
                    approxFilter = null;
                    flag2 = evaluate(obj, ac1, c1);
                }
                return flag2;
            }
            boolean flag = compile(dictionary, ac, j + 1);
            int i1 = currentIndex + 1;
            if(c == '&' || c == '|')
                do
                {
                    boolean flag1 = compile(dictionary, ac, i1);
                    i1 = currentIndex;
                    if(c == '&')
                        flag &= flag1;
                    else
                        flag |= flag1;
                    for(i1++; i1 < ac.length && Character.isWhitespace(ac[i1]); i1++);
                } while(i1 < ac.length && ac[i1] == '(');
            for(; i1 < ac.length && Character.isWhitespace(ac[i1]); i1++);
            if(i1 >= ac.length || ac[i1] != ')')
                throw new InvalidSyntaxException("missing ')' near " + i1, filter);
            currentIndex = i1;
            if(c == '!')
                return !flag;
            else
                return flag;
        }

        private boolean evaluate(Object obj, char ac[], char c)
        {
            if(obj == null)
                return false;
            Class class1 = obj.getClass();
            if(class1.isArray())
            {
                try
                {
                    for(int i = 0; i < Array.getLength(obj); i++)
                        if(evaluate(Array.get(obj, i), ac, c))
                            return true;

                }
                catch(Exception _ex) { }
                return false;
            }
            if(obj instanceof Vector)
            {
                Vector vector = (Vector)obj;
                for(Enumeration enumeration = vector.elements(); enumeration.hasMoreElements();)
                    if(evaluate(enumeration.nextElement(), ac, c))
                        return true;

                return false;
            } else
            {
                return evaluateSimple(obj, ac, c);
            }
        }

        private boolean evaluateSimple(Object obj, char ac[], char c)
        {
            if(obj == null)
                return false;
            String s = new String(ac);
            try
            {
                switch(c)
                {
                default:
                    break;

                case 62: // '>'
                    if((obj instanceof Number) || (obj instanceof Character) && ac.length == 1)
                        return numberCompare(obj, s) >= 0;
                    if(obj instanceof String)
                        return ((String)obj).compareTo(s) >= 0;
                    break;

                case 60: // '<'
                    if((obj instanceof Number) || (obj instanceof Character) && ac.length == 1)
                        return numberCompare(obj, s) <= 0;
                    if(obj instanceof String)
                        return ((String)obj).compareTo(s) <= 0;
                    // fall through

                case 61: // '='
                    if((obj instanceof Number) || (obj instanceof Character) && ac.length == 1)
                        return numberCompare(obj, s) == 0;
                    if(obj instanceof Boolean)
                    {
                        Boolean boolean1 = Boolean.valueOf(s);
                        return ((Boolean)obj).equals(boolean1);
                    } else
                    {
                        return starEquals(obj.toString(), ac);
                    }

                case 126: // '~'
                    if(obj instanceof Number)
                        return false;
                    if(approxFilter == null)
                    {
                        approxFilter = new char[ac.length + 2];
                        for(int i = 0; i < ac.length; i++)
                        {
                            approxFilter[i + 1] = ac[i];
                            if(approxFilter[i + 1] >= 'A' && approxFilter[i + 1] <= 'Z')
                                approxFilter[i + 1] += ' ';
                        }

                        approxFilter[0] = '*';
                        approxFilter[ac.length + 1] = '*';
                    }
                    return starEquals(obj.toString().toLowerCase(), approxFilter);
                }
            }
            catch(NumberFormatException _ex) { }
            return false;
        }

        private int numberCompare(Object obj, String s)
        {
            if(obj instanceof BigInteger)
                return ((BigInteger)obj).compareTo(new BigInteger(s));
            if(obj instanceof BigDecimal)
                return ((BigDecimal)obj).compareTo(new BigDecimal(s));
            if(obj instanceof Double)
            {
                double d = ((Number)obj).doubleValue();
                double d1 = (new Double(s)).doubleValue();
                if(d > d1)
                    return 1;
                return d != d1 ? -1 : 0;
            }
            if(obj instanceof Float)
            {
                float f = ((Number)obj).floatValue();
                float f1 = (new Float(s)).floatValue();
                if(f > f1)
                    return 1;
                return f != f1 ? -1 : 0;
            }
            if(obj instanceof Character)
            {
                char c = ((Character)obj).charValue();
                char c1 = s.charAt(0);
                if(c > c1)
                    return 1;
                return c != c1 ? -1 : 0;
            }
            long l = ((Number)obj).longValue();
            long l1 = Long.parseLong(s);
            if(l > l1)
                return 1;
            return l != l1 ? -1 : 0;
        }

        boolean starEquals(String s, char ac[])
        {
            char ac1[] = s.toCharArray();
            int i = -1;
            byte byte0 = -1;
            int j = 0;
            int k = 0;
            boolean flag = false;
            while(j < ac1.length)
            {
                if(k < ac.length && ac[k] == '\\')
                {
                    flag = true;
                    k++;
                }
                if(k < ac.length && ac[k] == '*' && !flag)
                {
                    i = k;
                    char c;
                    if(++k < ac.length)
                    {
                        c = ac[k];
                        if(c == '\\')
                            if(k + 1 < ac.length)
                            {
                                c = ac[++k];
                                flag = true;
                            } else
                            {
                                c = '\uFFFF';
                            }
                    } else
                    {
                        c = '\uFFFF';
                    }
                    for(; j < ac1.length && ac1[j] != c; j++);
                    if(j >= ac1.length)
                        break;
                }
                if(k >= ac.length || ac1[j] != ac[k])
                {
                    if(i == -1)
                        return false;
                    k = i;
                    flag = false;
                } else
                {
                    j++;
                    k++;
                    flag = false;
                }
            }
            return j == ac1.length && (k == ac.length || ac[k] == '*' && !flag);
        }

        String filter;
        int currentIndex;
        char approxFilter[];
    }
}


